/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.application.slash.converters.impl.enumChoice
import com.kotlindiscord.kord.extensions.commands.converters.impl.channel
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import com.kotlindiscord.kord.extensions.utils.hasPermission
import dev.kord.common.Color
import dev.kord.common.entity.Permission
import net.moonleay.lilJudd.data.tables.TimePlanningChannels
import net.moonleay.lilJudd.extensions.component.EnableOrDisable
import net.moonleay.lilJudd.extensions.component.FeatureEnum
import net.moonleay.lilJudd.util.Logger
import net.moonleay.lilJudd.util.MessageUtil
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.and
import org.jetbrains.exposed.sql.deleteWhere
import org.jetbrains.exposed.sql.insert
import org.jetbrains.exposed.sql.select
import org.jetbrains.exposed.sql.transactions.transaction

class FeatureManageExtension : Extension() {

    override val name = "feature"
    override val allowApplicationCommandInDMs: Boolean
        get() = false

    /*
    * Note: This has to be rewritten at some point to better support more features
    * and improve this mess of a class.
    * */
    override suspend fun setup() {
        publicSlashCommand(::FeatureManagerArgs) {
            name = "feature"
            description = "Manage features"

            this.action {
                val u = this.user
                if (!u.asMember(this.guild!!.id).hasPermission(Permission.Administrator)) {
                    this.respond {
                        embeds.add(
                            MessageUtil.getEmbed(
                                Color(0xE0311A),
                                "403: Forbidden",
                                "You cannot edit features, as you don't have the Administrator permission.",
                                u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                }
                val gID = this.guild!!.id.toString()
                val cID = this.arguments.channel.id.toString()
                val channel = this.arguments.channel
                val args = this.arguments
                Logger.out("${args.feature.readableName} ${args.setStatus.readableName} ${channel.data.name.value}")
                if (this.arguments.setStatus == EnableOrDisable.ENABLE) {
                    when (this.arguments.feature) {
                        FeatureEnum.TIMEPLANNINGFEATURE -> {
                            var alreadyExists = false
                            transaction {
                                alreadyExists = TimePlanningChannels.select {
                                    (TimePlanningChannels.channelid eq gID) and
                                            (TimePlanningChannels.channelid eq cID)
                                }.count() > 0
                            }
                            if (!alreadyExists) {
                                transaction {
                                    TimePlanningChannels.insert {
                                        it[TimePlanningChannels.serverid] = gID
                                        it[TimePlanningChannels.channelid] = cID
                                    } get TimePlanningChannels.id
                                }
                                this.respond {
                                    embeds.add(
                                        MessageUtil.getEmbed(
                                            Color(0x52E01A),
                                            "200: Success",
                                            "The feature was enabled in channel ${args.channel.data.name.value}",
                                            u.asUser().username + "#" + u.asUser().discriminator
                                        )
                                    )
                                }
                                return@action
                            }
                            this.respond {
                                embeds.add(
                                    MessageUtil.getEmbed(
                                        Color(0xE0311A),
                                        "403: Forbidden",
                                        "The feature is already enabled in this channel.",
                                        u.asUser().username + "#" + u.asUser().discriminator
                                    )
                                )
                            }
                        }


                        /*
                        FeatureEnum.PLANNINGNOTIFIER -> {
                            var alreadyExists = false
                            transaction {
                                alreadyExists = PlanningNotifierRoles.select {
                                    (PlanningNotifierRoles.serverid eq gID) and (PlanningNotifierRoles.channelid eq cID)
                                }.count() > 0
                            }
                            if (!alreadyExists) {

                                val hasTimeRole = this.guild!!.createRole {
                                    this.name = "available [${channel.data.name.value}]"
                                    this.mentionable = true
                                }
                                val htr = hasTimeRole.id.toString()

                                val wantsNotifsRole = this.guild!!.createRole {
                                    this.name = "notifications [${channel.data.name.value}]"
                                    this.mentionable = true
                                }
                                val wnr = wantsNotifsRole.id.toString()

                                transaction {
                                    PlanningNotifierRoles.insert {
                                        it[PlanningNotifierRoles.serverid] = gID
                                        it[PlanningNotifierRoles.channelid] = cID
                                        it[PlanningNotifierRoles.hastimeroleid] = htr
                                        it[PlanningNotifierRoles.wantstobenotifid] = wnr
                                    } get PlanningNotifierRoles.id
                                }

                                this.respond {
                                    embeds.add(
                                        MessageUtil.getEmbed(
                                            Color(0x52E01A),
                                            "200: Success",
                                            "The feature was enabled in channel ${args.channel.data.name.value} with roles ${hasTimeRole.mention} & ${wantsNotifsRole.mention}.",
                                            u.asUser().username + "#" + u.asUser().discriminator
                                        )
                                    )
                                }
                                return@action
                            }
                            this.respond {
                                embeds.add(
                                    MessageUtil.getEmbed(
                                        Color(0xE0311A),
                                        "403: Forbidden",
                                        "The feature is already enabled in this channel.",
                                        u.asUser().username + "#" + u.asUser().discriminator
                                    )
                                )
                            }
                        } */
                    }
                    return@action
                }
                //Disable
                when (this.arguments.feature) {
                    FeatureEnum.TIMEPLANNINGFEATURE -> {
                        var alreadyExists = false
                        transaction {
                            alreadyExists = TimePlanningChannels.select {
                                (TimePlanningChannels.serverid eq gID) and
                                        (TimePlanningChannels.channelid eq cID)
                            }.count() > 0
                        }
                        if (alreadyExists) {
                            transaction {
                                val matchingEntries = TimePlanningChannels.select {
                                    (TimePlanningChannels.serverid eq gID) and
                                            (TimePlanningChannels.channelid eq cID)
                                }.toList()

                                matchingEntries.forEach { entry ->
                                    TimePlanningChannels.deleteWhere { id eq entry[id] }
                                }
                            }
                            this.respond {
                                embeds.add(
                                    MessageUtil.getEmbed(
                                        Color(0x52E019),
                                        "200: Success",
                                        "The feature was disabled.",
                                        u.asUser().username + "#" + u.asUser().discriminator
                                    )
                                )
                            }
                            return@action
                        }
                        this.respond {
                            embeds.add(
                                MessageUtil.getEmbed(
                                    Color(0xE0311A),
                                    "403: Forbidden",
                                    "The feature is already disabled in this channel.",
                                    u.asUser().username + "#" + u.asUser().discriminator
                                )
                            )
                        }
                    }

                    /*
                    FeatureEnum.PLANNINGNOTIFIER -> {
                        var alreadyExists = false
                        transaction {
                            alreadyExists = PlanningNotifierRoles.select {
                                (PlanningNotifierRoles.serverid eq gID) and (PlanningNotifierRoles.channelid eq cID)
                            }.count() > 0
                        }
                        if (alreadyExists) {
                            var matchingEntries: List<ResultRow> = mutableListOf()
                            transaction {
                                matchingEntries = PlanningNotifierRoles.select {
                                    (PlanningNotifierRoles.serverid eq gID) and
                                            (PlanningNotifierRoles.channelid eq cID)
                                }.toList()
                            }
                            for (e in matchingEntries) {
                                this.guild!!.getRoleOrNull(Snowflake(e[PlanningNotifierRoles.hastimeroleid]))?.delete()
                                this.guild!!.getRoleOrNull(Snowflake(e[PlanningNotifierRoles.wantstobenotifid]))
                                    ?.delete()
                            }

                            transaction {
                                matchingEntries.forEach { entry ->
                                    PlanningNotifierRoles.deleteWhere { id eq entry[id] }
                                }
                            }
                            this.respond {
                                embeds.add(
                                    MessageUtil.getEmbed(
                                        Color(0x52E019),
                                        "200: Success",
                                        "The feature was disabled.",
                                        u.asUser().username + "#" + u.asUser().discriminator
                                    )
                                )
                            }
                            return@action
                        }
                        this.respond {
                            embeds.add(
                                MessageUtil.getEmbed(
                                    Color(0xE0311A),
                                    "403: Forbidden",
                                    "The feature is already disabled in this channel.",
                                    u.asUser().username + "#" + u.asUser().discriminator
                                )
                            )
                        }
                    } */
                }
            }
        }
    }

    inner class FeatureManagerArgs : Arguments() {

        val feature by enumChoice<FeatureEnum> {
            this.name = "feature"
            this.description = "The targeted feature"
            this.typeName = "en_US"
        }

        val setStatus by enumChoice<EnableOrDisable> {
            this.name = "set"
            this.description = "Set enabled or disabled"
            this.typeName = "en_US"
        }

        val channel by channel {
            this.name = "channel"
            this.description = "Target Channel"
        }
    }
}
