/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.features

import dev.inmo.krontab.buildSchedule
import dev.inmo.krontab.doInfinity
import dev.kord.common.Color
import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.channel.createMessage
import dev.kord.core.entity.channel.MessageChannel
import dev.kord.rest.builder.message.create.actionRow
import kotlinx.coroutines.delay
import net.moonleay.lilJudd.Bot
import net.moonleay.lilJudd.data.entry.PlanningNotifierRolesData
import net.moonleay.lilJudd.data.tables.PlanningNotifierRoles
import net.moonleay.lilJudd.data.tables.TimePlanningChannels
import net.moonleay.lilJudd.util.ButtonUtil
import net.moonleay.lilJudd.util.Logger
import net.moonleay.lilJudd.util.MessageUtil
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.transactions.transaction
import java.time.ZoneId
import java.time.ZonedDateTime


object TimeManager {
    /*  /--------------- Seconds
        | /------------- Minutes
        | | /----------- Hours
        | | | /--------- Days of months
        | | | | /------- Months
        | | | | | /----- (optional) Year
        | | | | | | /--- (optional) Timezone offset
        | | | | | | |  / (optional) Week days
        * * * * * * 0o *w*/

    suspend fun registerThread() {
        Logger.out("Adding message scheduler...")
        val scheduler = buildSchedule("0 0 4 * * * 0o 1w") // 0 0 4 * * * 0o 1w // 0o is UTC
        scheduler.doInfinity {
            Logger.out("Starting to notify...")


            // ChannelID, ServerID
            val channelList = mutableMapOf<Snowflake, Snowflake>()
            // ChannelID, Data
            val roleList = mutableMapOf<Snowflake, PlanningNotifierRolesData>()

            transaction {
                for (tp in TimePlanningChannels.selectAll()) {
                    channelList[Snowflake(tp[TimePlanningChannels.channelid])] =
                        Snowflake(tp[TimePlanningChannels.serverid])
                    Logger.out("Have to notify channel with ID ${tp[TimePlanningChannels.channelid]}.")
                }

                for (pnr in PlanningNotifierRoles.selectAll()) {
                    roleList[Snowflake(pnr[PlanningNotifierRoles.channelid])] = PlanningNotifierRolesData(
                        pnr[PlanningNotifierRoles.serverid],
                        pnr[PlanningNotifierRoles.channelid],
                        pnr[PlanningNotifierRoles.hastimeroleid],
                        pnr[PlanningNotifierRoles.wantstobenotifid]
                    )
                    Logger.out("Have to ping roles: ${pnr[PlanningNotifierRoles.wantstobenotifid]}}")
                }
            }
            Logger.out("${channelList.count()} Channels to notify with ${roleList.count()} Roles to ping!")
            for (ch in channelList.keys) {
                if (Bot.bot.kordRef.getChannel(ch) == null)
                    continue // TODO: Check if the channel is valid in another shard
                val c = Bot.bot.kordRef.getChannelOf<MessageChannel>(ch)!!
                c.createMessage {
                    if (roleList[ch] != null) {
                        this.content = "<@&${Snowflake(roleList[ch]?.wantstobenotifid!!)}>"
                    }
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            Color(0X4C4645),
                            "Time Planning Feature",
                            "Do you have time on the following Days?",
                            "Automated Message"
                        )
                    )
                }
                delay(2000)
                var then = ZonedDateTime.now(ZoneId.of("Europe/Berlin")).withHour(4).withMinute(0).withSecond(0)
                repeat(7) {
                    c.createMessage {
                        this.embeds.add(
                            MessageUtil.getEmbedWithTable(
                                Color(0X4C4645),
                                "",
                                "${then.dayOfWeek.name}, ${then.dayOfMonth}.${then.monthValue}.${then.year} /${it + 1}. weekday",
                                mapOf(
                                    "Is available" to listOf(),
                                    "May be available" to listOf(),
                                    "Is not available" to listOf()
                                )
                            )
                        )

                        this.actionRow {
                            this.components.addAll(ButtonUtil.getTimePlannerButtons().components)
                        }
                    }
                    then = then.plusDays(1).withHour(4).withMinute(0).withSecond(0)
                    Logger.out("Finished sending day $it")

                    delay(1000)
                }
                Logger.out("Finished with ${c.data.guildId.value}")
            }
            Logger.out("Done! Until next Monday! <3   ")
        }
    }
}
